declare module BA.Ui.Controls {
    /**
     *
     **/
    class DurationControlEvents {
        /** internal flag indicating that the current process happens during a self-triggered change event */
        /**
         * Initializes the duration info model in the hidden field, if possible (i.e. not in read-mode).
         *
         * @param sender the duratioon control
         * @param evt event object
         */
        static Init(sender: any, evt: any): void;
        /**
         * Event function that is triggered on two occasions:
         * 1. When the user changes anything during editing the start date field (roll some part of the date, select
         *      another date, etc)
         * 2. When the actual changed value is applied (when the user leaves the field)
         * The difference here is, that in case 1. the control (sender) still has its old value and does not reflect
         * the changes yet, which the user has made. As we want a life-update of the end date field, we have to fire
         * an OnChange-event on the control manually (and make sure, it does not loop).
         * The mentioned life-update itself consists of the end date moving to a new date while maintaining the same
         * duration.
         *
         * @param sender the start date control
         * @param evt event object
         */
        static OnStartChanged(sender: ASPxClientDateEdit, evt: any): void;
        /**
         * Event function that is triggered on two occasions:
         * 1. When the user changes anything during editing the end date field (roll some part of the date, select
         *      another date, etc)
         * 2. When the actual changed value is applied (when the user leaves the field)
         * The difference here is, that in case 1. the control (sender) still has its old value and does not reflect
         * the changes yet, which the user has made. As we want a life-update of the current duration, we have to fire
         * an OnChange-event on the control manually (and make sure, it does not loop).
         * The mentioned life-update itself consists of changing the duration to the new difference between start and
         * end date, while preventing the end date to be before the start date.
         *
         * @param sender the end date control
         * @param evt
         */
        static OnEndChanged(sender: ASPxClientDateEdit, evt: any): void;
        /**
         * Event function that is called when the all-day-checkbox is clicked (prior to partial refresh).
         * If the user jsut switches to all-day-mode, it stores the current time portions of the dates, so the server
         * can recall them, should the user change their mind and switch back.
         *
         * @param sender the all-day-checkbox
         * @param evt the event object
         */
        static OnAllDayChanged(sender: ASPxClientCheckBox, evt: any): void;
        static PrepareDuration(sender: ASPxClientDateEdit): void;
        /**
         * Helper function to find the DurationInfoModel (the duration control helper field) from the given control.
         * It climbs up in control structure until it finds the actual duration control and its hidden companion.
         * If it was found, it deserializes its contents and returns them as DurationInfoModel object.
         *
         * @param control the control to start the search from
         * @returns the found DurationInfoModel object or null
         */
        private static GetDurationModel;
        /**
         * Helper function that serializes the given DurationInfoModel to the control's helper field.
         * No need to search for the helper field here: The model contains the name of the duration control.
         *
         * @param model the DurationInfoModel to store
         */
        private static SetDurationModel;
        /**
         * (Re)calculate Duration of DurationInfoModel from start/end controls.
         * @param model
         */
        private static CalcDuration;
    }
}
