declare module BA.Ui.TabController {
    class TabTools {
        private static tt_showtimeout;
        private static tt_hidetimeout;
        private static _WaitAnimationVisible;
        private static _WaitAnimationCounter;
        private static _WaitAnimationTimeout;
        private static _PreventNextEscTabClose;
        private static activeElement;
        /**
         * Opens a new tab
         * @param url:                  The url: objectopen (or download)
         * @param asDirty:              Open the tab dirty? (Defaul: false)
         * @param overrideTabReference: Override the tab reference
         * @param tabOpenedCallback:    Callback function: executed only if a new tab was opened (will past the new tab)
         * @param predecessor:          Specifies the tab next to which the new tab will be opened (Default null opens next to active tab)
         */
        static OpenTab(url: string, asDirty?: Boolean, overrideTabReference?: string, tabOpenedCallback?: (tab: Tab) => void, asLastVisibleTab?: boolean, isUniqueTab?: boolean, predecessor?: Tab): void;
        /**
         * Opens url in the tab where this is called
         * @param url: The url
         * @param overrideTabReference: Override the tab reference
         */
        static OpenInSameTab(url: string, overrideTabReference?: string): void;
        /**
         * Opens url in the tab where this is called unless it is pinned. If pinned this opens a new tab
         * @param url:                  The url: objectopen (or download)
         * @param asDirty:              Open the tab dirty? (Defaul: false and only for new tabs)
         * @param overrideTabReference: Override the tab reference
         */
        static OpenInSameTabIfNotPinned(url: string, asDirty: boolean, overrideTabReference: string): void;
        static OpenNewTab(predecessor?: Tab): void;
        /**
         * Sets the flag to prevent a tab close by ESC pressed to true
         *
         * @param value optional value whether to prevent the next esc or not (defaults to true). This is needed when
         *              there is a period of time where the escape key should not close the tab but it isn't
         *              necessarily pressed during this period.
         */
        static PreventNextEscTabClose(value?: boolean): void;
        /**
         * Updates the anchors in html to be handled by the proper methods in the system
         */
        static UpdateLinks(): void;
        /**
         * Clears a tab (sets as not dirty) and closes it.
         * By clearing it, it avoids the need of confirmation from the user to close the tab.
         */
        static ForceTabClose(): void;
        /**
         * Performs the next steps to be done after receiving a form result
         *
         * @param jsonFormResult: form result in json format
         */
        static HandleJsonFormResult(jsonFormResult: Models.JsonFormResult): void;
        /**
         * Checks if a tab or the active tab can be closed
         *
         * @param guid: Identifier of the tab to check. If null, the active tab will be checked.
         */
        static GetTabCanClose(guid?: string): boolean;
        /**
         * Activates the corresponding tab through the controller.ts instance.
         *
         * @param guid: identifier of the tab to activate
         */
        static ActivateTab(guid: string): void;
        /**
         * Marks the current tab as loaded through the controller.ts instance
         */
        static MarkTabAsLoaded(): void;
        /**
         * Sets the url of the current tab through the controller.ts instance
         *
         * @param url: The url for the tab
         */
        static SetTabUrl(url: string): void;
        /**
         * Replaces the url of the current tab through the controller.ts instance
         *
         * @param url: The new url for the tab
         */
        static ReplaceTabUrl(url: string): void;
        /**
         * Sets the reference of the current tab through the controller.ts instance
         * The reference of the tab is used as unique identifier for opened tabs
         *
         * @param ref: The reference for the tab
         */
        static SetTabReference(ref: string): void;
        static AttachToActivatedEvent(activateFunc: Function): void;
        /**
         * Sets the record oid for the current tab through the controller.ts instance
         *
         * @param oid: record id to set
         */
        static SetDocumentOidForTab(oid: string): void;
        /**
         * Updates parts of a given document given the document id and the ids of the parts to update, through the controller.ts instance
         *
         * @param oid: id of the document that needs to have parts updated
         * @param keys: keys of the parts to be updated
         */
        static UpdateDocumentPartsForTab(oid: string, keys: string[]): void;
        /**
         * Updates grid views of a given tab and the ids of the grid views to update
         *
         * @param guid: id of the tab
         * @param gridUINames: ids of the grids to be updated
         * @param notFoundMessage: optional custom message if the tab is not found
         */
        static UpdateGridViewsInTab(guid: string, gridUINames: string[], notFoundMessage?: string, andActivate?: boolean): void;
        /**
         * Given a tab guid and a callback function, this method executes the callback function once the tab with the corresponding guid exists.
         * For that, this methods checks the existence every 100 milliseconds with a limit equal to maxRetries
         *
         * @param tabGuid: the guid of the tab that should exist to execute the callback function
         * @param maxRetries: the maximum number of retries
         * @param callback: the callback function to be executed
         */
        static WaitForTabLoaded(tabGuid: string, maxRetries: number, callback: Function): void | boolean;
        /**
         * Sets the title of the current tab through the controller.ts instance, or only in the browser tab if there is no controller instance
         *
         * @param tabTitle: new title
         */
        static SetTabName(tabTitle: string): void;
        /**
         * Sets the icon of the current tab through the controller.ts instance
         *
         * @param icon: new icon
         */
        static SetTabIcon(icon: string): void;
        /**
         * Gets the tab title of the current tab through the controller.ts instance
         */
        static GetTabName(): string;
        /**
         * Gets the tab id of the current tab
         */
        static GetTabId(): string;
        /**
         * Checks if a tab is pinned through the controller.ts instance
         *
         * @param guid: guid of the tab to check. If null, will check the current tab
         */
        static IsTabPinned(guid?: string): boolean;
        /**
         * Checks if the active tab is pinned through the controller.ts instance
         */
        static IsActiveTabPinned(): boolean;
        /**
         * Marks a tab as dirty through the controller.ts instance.
         * This should not be used from the edit events of controls. For that use, refer to:
         * BA.Ui.Controls.ControlEvents.SetTabAsDirty
         *
         * @param tab: Tab to mark as dirty
         */
        static SetTabAsDirty(tab: TabController.Tab): void;
        /**
         * Checks if a tab is marked as dirty through the controller.ts instance
         *
         * @param guid: guid of the tab to check. If null, will check the current tab
         */
        static GetTabIsDirty(guid?: string): boolean;
        /**
         * If corresponds, executes the autorefresh of the current tab through the controller.ts instance
         */
        static ExecuteAutoRefresh(): void;
        /**
         * Sets the current tab as clear (not dirty) through the controller.ts instance
         */
        static SetTabAsClear(): void;
        /**
         * Sets all the tab as clear (not dirty) through the controller.ts instance
         */
        static ClearAllTabs(): void;
        /**
         * Sets the autorefresh of the tab from where the current tab was opened as true or false through the controller.ts instance
         *
         * @param autoRefresh: True if the tab should be autorefreshed on activate. False otherwise
         */
        static SetParentTabAutorefresh(autoRefresh: boolean): void;
        /**
         * Sets the autorefresh of the current tab as true or false through the controller.ts instance
         *
         * @param autoRefresh: True if the tab should be autorefreshed on activate. False otherwise
         */
        static SetTabAutorefresh(autoRefresh: boolean): void;
        /**
         * Sets if the autorefresh of the current tab should be forced or not, even if the tab would not normally auto refresh.
         * Done through the controller.ts instance
         *
         * @param autoRefresh: True if the tab should be autorefreshed on activate. False otherwise
         */
        static SetForcedTabAutorefresh(forcedAutoRefresh: boolean): void;
        /**
         * Checks if the current tab is marked as autorefresh.
         * Done through the controller.ts instance
         */
        static GetTabAutorefresh(): boolean;
        /**
         * Checks if the autorefresh of the current tab should be forced or not, even if the tab would not normally auto refresh.
         * Done through the controller.ts instance
         */
        static GetForcedTabAutorefresh(): boolean;
        /**
         * Shows the loading animation through the controller.ts instance. If marked as delayed, it waits few milliseconds before showing the animation
         *
         * @param delayed: if the animation should be delayed. Default: true
         */
        static ShowWaitAnimation(delayed?: boolean, inIFrame?: boolean): void;
        /**
         * Hides the loading animation through the controller.ts instance.
         */
        static HideWaitAnimation(): void;
        /**
         * Shows the loading animation in the iframe
         */
        static ShowInIFrameWaitAnimation(): void;
        /**
         * Hides the loading animation in the iframe
         */
        static HideInIFrameWaitAnimation(context?: any): void;
        /**
         * Checks if there is any loading animation visible or waiting to be shown, both in the controller.ts instance or in the iframe through this class
         */
        static IsWaitAnimationVisibleOrLoading(): boolean;
        /**
         * Shows a blocker element in the ui through the controller.ts instance to avoid interaction from the user
         */
        static ShowInputBlocker(): void;
        /**
         * Hides the blocker element in the ui through the controller.ts instance
         */
        static HideInputBlocker(): void;
        /**
         * Checks if the input blocker is shown. I.e. if the user cannot interact. True means the input is blocked and there should be no interaction with the ui
         */
        static IsInputBlocked(): boolean;
        static CloseAndOpenTab(url: string): void;
        /**
         * Closes a tab given its identifier
         *
         * @param guid: guid for the tab that has to be closed
         */
        static CloseTab(guid?: string, autoUpdateTabBar?: boolean): void;
        /**
         * Update the tab bar in case any tab was removed. Done through the controller.ts instance
         */
        static UpdateTabBarIfTabsRemoved(): void;
        /**
        * Shows the tooltip
        *
        * @param sender: Hovered html element
        * @param event: Event (for mouse position)
        */
        static ShowTooltip(sender: JQuery, event: JQueryEventObject): void;
        /**
        * Hides the tooltip, if visible. Cancel any pending show otherwise.

        * @param immediate: Avoids keeping the tooltip for a short delay
        */
        static HideTooltip(sender: JQuery, immediate?: boolean): void;
        /**
         * Cancels a tooltip hide, that already started
         *
         * @param sender: The tooltip div
         */
        static CancelHideTootip(sender: JQuery): void;
        /**
         * Cancels a tooltip show, that already started
         *
         * @param sender: The tooltip div
         */
        static CancelShowTootip(): void;
        /**
         * Controls the event keydown and does the corresponding tasks depending on the pressed key
         *
         * @param e: event parameter
         */
        private static KeydownHandler;
        private static SomethingInGridOpened;
        private static SpecialTabOpened;
        /**
         * Attaches the keydown handler, so the system starts processing key presses
         */
        static AttachKeydownHandler(): void;
        /**
         * Dettaches the keydown handler, so the system does not process key presses
         */
        static DetachKeydownHandler(): void;
        /**
         * Returns the hidden data of the currently opened dialog or of the tab form.
         */
        static GetCurrentFormHiddenData(control: ASPxClientControl): BA.Ui.Models.FormHiddenDataModel;
        /**
         * Returns the hidden data of the form
         */
        static GetFormHiddenData(): BA.Ui.Models.FormHiddenDataModel;
        /**
         * Returns the form name of the currently opened dialog or of the tab form.
         */
        static GetCurrentFormName(control: ASPxClientControl): string;
        /**
         * Sets the edit mode in the form hidden data as true or false.
         * If true, the form will be loaded in edit mode in the next load (next post). It will be loaded in read mode otherwise
         *
         * @param enabled: the wanted value for the edit mode
         */
        static SetEditModeStatus(enabled: boolean): void;
        /**
         * Gets the current edit mode status from the form hidden data
         */
        static GetEditModeStatus(): boolean;
        /**
         * Sets the value of the must save property in the form hidden data.
         * If true, the data of the form will be saved during the next post.
         *
         * @param mustSave: the wanted value for the must save property
         */
        static SetMustSave(mustSave: boolean): void;
        /**
          * Sets the corresponding property the form hidden data as true or false.
          * If true, the form will be loaded in read mode in the next load (next post), regardless of the edit mode property
          *
          * @param force: the wanted value for the force edit mode
          */
        static SetMustForceReadMode(force: boolean): void;
        /** Returns the active tab's guid, null if not possible. */
        static GetActiveTabGuid(): BA.Classes.Guid;
        /** Returns the guid of the current context's tab, null if not possible. */
        static GetCurrentTabGuid(): BA.Classes.Guid;
        /** Returns the current context's tab, null if not possible. */
        static GetCurrentTab(): BA.Ui.TabController.Tab;
        /**
         * Function that is executed, when a tab is loaded (was: $(document).ready).
         **/
        static InitTabWhenLoaded(): void;
        /**
         * Function to open tabs or dialogs from links generated with BALinkRecord()
         **/
        static OpenLinkRecord(param: string): void;
        /**
         * Refreshes the current or given tab.
         * @param guid Guid of the tab to refresh or empty for current tab.
         */
        static RefreshTab(guid?: string): void;
        /**
        * Clone an ORM
        * @param currentRecordOid
        * */
        static CopyOrmSelectedInGrid(grid: BAClientGridBase, customData: CustomData): void;
        /**
        * Clone an ORM
        * @param currentRecordOid
        * */
        static CopyOrmOutOfForm(currentRecordOid: string, formId: string, customData: CustomData): void;
        static HighlightBrowserTab: (blinkText: string) => void;
    }
}
